# Copyright (c) 2002 Cunningham & Cunningham, Inc.
# Released under the terms of the GNU General Public License version 2 or later.

require "test/unit"

require "fit/Fixture"
require "fit/Parse"

module Fit

  class FrameworkTest < Test::Unit::TestCase


    def test_parsing
      p = Fit::Parse.new("leader<Table foo=2>body</table>trailer", ["table"])
      assert_equal("leader", p.leader)
      assert_equal("<Table foo=2>", p.tag)
      assert_equal("body", p.body)
      assert_equal("trailer", p.trailer)
    end

   def test_recursing
     p = Fit::Parse.new("leader<table><TR><Td>body</tD></TR></table>trailer")
     assert_equal(nil, p.body)
     assert_equal(nil, p.parts.body)
     assert_equal("body", p.parts.parts.body)
   end


   def test_iterating
     p = Parse.new("leader<table><tr><td>one</td><td>two</td><td>three</td></tr></table>trailer")
     assert_equal("one", p.parts.parts.body)
     assert_equal("two", p.parts.parts.more.body)
     assert_equal("three", p.parts.parts.more.more.body)
   end


   def test_indexing
     p = Parse.new("leader<table><tr><td>one</td><td>two</td><td>three</td></tr><tr><td>four</td></tr></table>trailer")
     assert_equal("one",   p.at(0,0,0).body)
     assert_equal("two",   p.at(0,0,1).body)
     assert_equal("three", p.at(0,0,2).body)
     assert_equal("three", p.at(0,0,3).body)
     assert_equal("three", p.at(0,0,4).body)
     assert_equal("four",  p.at(0,1,0).body)
     assert_equal("four",  p.at(0,1,1).body)
     assert_equal("four",  p.at(0,2,0).body)
     assert_equal(1, p.size())
     assert_equal(2, p.parts.size())
     assert_equal(3, p.parts.parts.size())
     assert_equal("one",   p.leaf().body)
     assert_equal("four",  p.parts.last().leaf().body)
   end

   def test_parse_exception
     Parse.new("leader<table><tr><th>one</th><th>two</th><th>three</th></tr><tr><td>four</td></tr></table>trailer")
     fail("expected exception not thrown")
   rescue Fit::ParseException => e
     assert_equal(17, e.error_offset)
     assert_equal("Can't find tag: td", e.message)
   end

   def test_text
     tags = [ "td" ]
     p = Parse.new("<td>a&lt;b</td>", tags)
     assert_equal("a&lt;b", p.body)
     assert_equal("a<b", p.text())

     p = Parse.new("<td>\ta&gt;b&nbsp;&amp;&nbsp;b>c &&&nbsp;</td>", tags)
     assert_equal("a>b & b>c &&", p.text())
     p = Parse.new("<TD><P><FONT FACE=\"Arial\" SIZE=2>GroupTestFixture</FONT></TD>", tags)
     assert_equal("GroupTestFixture",p.text())
   end

    def test_unescape
      assert_equal("a<b",          Parse.unescape("a&lt;b"))
      assert_equal("a>b & b>c &&", Parse.unescape("a&gt;b&nbsp;&amp;&nbsp;b>c &&"))
    end

    def test_unformat
      assert_equal("ab",  Parse.unformat("<font size=+1>a</font>b"))
      assert_equal("ab",  Parse.unformat("a<font size=+1>b</font>"))
      assert_equal("a<b", Parse.unformat("a<b"))
    end

=begin
    public void testTypeAdapter () throws Exception {
        TestFixture f = new TestFixture ();
        TypeAdapter a = TypeAdapter.on(f, f.getClass().getField("sampleInt"));
        a.set(a.parse("123456"));
        assert_equal(123456, f.sampleInt);
        assert_equal("-234567", a.parse("-234567").toString());
        a = TypeAdapter.on(f, f.getClass().getField("sampleInteger"));
        a.set(a.parse("54321"));
        assert_equal("54321", f.sampleInteger.toString());
        a = TypeAdapter.on(f, f.getClass().getMethod("pi", new Class[] {}));
        assert_equal(3.14159, ((Double)a.invoke()).doubleValue(), 0.00001);
        assert_equal(new Double(3.14159862), a.invoke());
        a = TypeAdapter.on(f, f.getClass().getField("ch"));
        a.set(a.parse("abc"));
        assert_equal('a', f.ch);
        a = TypeAdapter.on(f, f.getClass().getField("name"));
        a.set(a.parse("xyzzy"));
        assert_equal("xyzzy", f.name);
        a = TypeAdapter.on(f, f.getClass().getField("sampleFloat"));
        a.set(a.parse("6.02e23"));
        assert_equal(6.02e23, f.sampleFloat, 1e17);
        a = TypeAdapter.on(f, f.getClass().getField("sampleArray"));
        a.set(a.parse("1,2,3"));
        assert_equal(1, f.sampleArray[0]);
        assert_equal(2, f.sampleArray[1]);
        assert_equal(3, f.sampleArray[2]);
        assert_equal("1,2,3", a.toString(f.sampleArray));
        assertTrue(a.equals(new int [] {1,2,3}, f.sampleArray));
        a = TypeAdapter.on(f, f.getClass().getField("sampleDate"));
        Date date = new Date(49,4,26);
        a.set(a.parse(DateFormat.getDateInstance().format(date)));
        assert_equal(date, f.sampleDate);
        a = TypeAdapter.on(f, f.getClass().getField("sampleByte"));
        a.set(a.parse("123"));
        assert_equal(123, f.sampleByte);
        a = TypeAdapter.on(f, f.getClass().getField("sampleShort"));
        a.set(a.parse("12345"));
        assert_equal(12345, f.sampleShort);
    }

    class TestFixture extends ColumnFixture { // used in testTypeAdapter
        public byte sampleByte;
        public short sampleShort;
        public int sampleInt;
        public Integer sampleInteger;
        public float sampleFloat;
        public double pi () {return 3.14159862;}
        public char ch;
        public String name;
        public int[] sampleArray;
        public Date sampleDate;

    }

    public void testScientificDouble() {
        Double pi = new Double(3.141592865);
        assert_equal(ScientificDouble.valueOf("3.14"), pi);
        assert_equal(ScientificDouble.valueOf("3.141"), pi);
        assert_equal(ScientificDouble.valueOf("3.1415"), pi);
        assert_equal(ScientificDouble.valueOf("3.14159"), pi);
        assert_equal(ScientificDouble.valueOf("3.141592865"), pi);
        assertTrue(!ScientificDouble.valueOf("3.140").equals(pi));
        assertTrue(!ScientificDouble.valueOf("3.144").equals(pi));
        assertTrue(!ScientificDouble.valueOf("3.1414").equals(pi));
        assertTrue(!ScientificDouble.valueOf("3.141592863").equals(pi));
        Float av = new Float(6.02e23);
        assert_equal(ScientificDouble.valueOf("6.0e23"), av);
    }
=end

    def test_escape
      junk = "!@#$%^*()_-+={}|[]\\:\";',./?`"
      assert_equal(junk, Fixture.escape(junk))
      assert_equal("", Fixture.escape(""))
      assert_equal("&lt;", Fixture.escape("<"))
      assert_equal("&lt;&lt;", Fixture.escape("<<"))
      assert_equal("x&lt;", Fixture.escape("x<"))
      assert_equal("&amp;", Fixture.escape("&"))
      assert_equal("&lt;&amp;&lt;", Fixture.escape("<&<"))
      assert_equal("&amp;&lt;&amp;", Fixture.escape("&<&"))
      assert_equal("a &lt; b &amp;&amp; c &lt; d", 
		   Fixture.escape("a < b && c < d"));
    end


end
end
